/*****************************************************************************
 *  INCLUDE FILE(S)
 *  ---------------
 *	This .cpp file should automatically include all the support files needed
 *	for this particular class. In addition, this file may include other .h
 *	files defining additional callback interfaces for use by a third party.   
 ****************************************************************************/ 
#ifndef INITGUID        /* INITGUID causes the actual GUIDs to be declared, */ 
	#define INITGUID      /* not just references to them. Must be declared in */
#endif                  /* only one file. */          

#ifndef _H_cregster
	#include "PsxReg.h"
#endif

#include <string.h> 

/*****************************************************************************
 *  CLASS INTERFACE(S)
 *  ------------------
 *  The interface(s) implemented by your MOA class are specified here.  Note
 *	that at least one class in your Xtra should implement the IMoaRegister
 *	interface.
 *  NOTE: Because C++ does not use a lpVtbl to reference an interface's
 *	methods, the actual method declaration is done in the .h file.
 *
 *  Syntax:
 *  BEGIN_DEFINE_CLASS_INTERFACE(<class-name>, <interface-name>) 
 ****************************************************************************/ 
BEGIN_DEFINE_CLASS_INTERFACE(CHxRegister, IMoaRegister) 
END_DEFINE_CLASS_INTERFACE


/*******************************************************************************
 * SCRIPTING XTRA MESSAGE TABLE DESCRIPTION.
 *
 * The general format is:
 * xtra <nameOfXtra>
 * new object me [ args ... ]
 * <otherHandlerDefintions>
 * --
 * The first line must give the name of the Scripting xtra.
 * The remaining lines give the names of the handlers that this xtra implements
 * along with the required types of the arguments for each handler.
 * 
 * -- Pre-defined handler new 
 * The new handler will be called when a child object is created, 
 * the first argument is always the child object and you defined any remaining arguments.
 * The new handler is the place to initialize any memory required by the child object.
 * 
 * -- Simple Handler Definitions
 * Each handler definition line is format like this:
 * <handlerName> <argType1> <argName1>, <argType2> <argName2> ...
 * The first word is the handler name. Following this are types description for
 * the argument to the handler, each separated by a comma.
 * The argument name <argName>, may be omited.
 * Permited argument types are: 
 * 	integer 
 * 	float
 * 	string
 * 	symbol
 * 	object
 * 	any
 * 	*
 * For integer, float, string, symbol, and object, the type of the argument must 
 * match. The type any means allow any type. The asterisk (*) means any number and 
 * any type of arguments.
 * 
 * The first argument is the child object and is always declared to be of type object.
 * 
 * -- Global Handlers
 * An asterisk (*) preceeding the handler name signifies a global handler.
 * This handler is at the global scope level and can be called from any
 * movie.
 * 
 * -- Xtra level handlers
 * A plus (+) preceeding the handler name signifies an Xtra level handler. 
 * This kind of handler can be called directly from the Xtra reference,
 * without creating a child object.
 * 
 * The enumerated list in beeplngo.H must correspond directly with the msgTable 
 * (i.e. they must be in the same order).
 * 
 *******************************************************************************/ 
 
 /* MODIFY:  */

const static char msgTable1[] = {
	"xtra PDFCreator\n"
	"-- PDFCreator v0.8.1 (c) 2013 Valentin Schmidt\n"
	"-- contact: fluxus@freenet.de\n\n"
	"new object me\n" //string
	"\n----------------------------------------\n"
	"-- General Functions\n"
	"----------------------------------------\n"
	"pdf_set_parameter object me, string, string\n" //*
	"pdf_set_value object me, string, float\n" //*
	"pdf_get_value object me, string, *\n" //*
	"pdf_open object me, string\n"
	"pdf_close object me\n"
	"pdf_begin_page object me, *\n"
	"pdf_end_page object me\n"
	"\n----------------------------------------\n"
	"-- Text Functions\n"
	"----------------------------------------\n"
	"pdf_findfont object me, string, string, integer\n"
	"pdf_setfont object me, integer, float\n"
	"pdf_set_text_pos object me, integer, integer\n"
	"pdf_show object me, string\n"
	"pdf_show_xy object me, string, float, float\n"
	"pdf_continue_text object me, string\n"
	"pdf_show_boxed object me, string, float, float, float, float, string, string\n"
	"pdf_stringwidth object me, string, integer, float\n"
	"\n----------------------------------------\n"
	"-- Color Functions\n"
	"----------------------------------------\n"
	"pdf_setcolor object me, string, string, float, float, float, float\n"
	"pdf_makespotcolor object me, string, integer\n"
	"pdf_begin_pattern object me, float, float, float, float, integer\n" // ABSTURZ
	"pdf_end_pattern object me\n"
	"\n----------------------------------------\n"
	"-- Image Functions\n"
	"----------------------------------------\n"
	"pdf_open_image_file object me, string, string, string, integer\n"
	"pdf_open_image object me, object\n"
	"pdf_place_image object me, integer, float, float, float\n"
	"pdf_add_image_file object me, string, string, float, float, float\n"
	"pdf_add_image object me, object, float, float\n"
	"pdf_close_image object me, integer\n"
	"pdf_begin_template object me, float, float\n"
	"pdf_end_template object me\n"
	"\n----------------------------------------\n"
	"-- Hypertext Functions\n"
	"----------------------------------------\n"
	"pdf_set_info object me, string, string\n"
	"pdf_add_locallink object me, float, float, float, float, int, string\n"
	"pdf_add_weblink object me, float, float, float, float, string\n"
	"pdf_add_launchlink object me, float, float, float, float, string\n"
	"pdf_add_pdflink object me, float, float, float, float, string, integer, string\n"
	"pdf_add_note object me, float, float, float, float, string, string, string, integer\n"
	"pdf_add_bookmark object me, string, integer, integer\n"
	"pdf_set_border_style object me, string, float\n"
	"pdf_set_border_color object me, float, float, float\n"
	"pdf_set_border_dash object me, float, float\n"
	"pdf_attach_file object me, float, float, float, float, string, string, string, string, string\n"
	"pdf_add_thumbnail object me, integer\n"
	"\n----------------------------------------\n"
	"-- Graphics Functions\n"
	"----------------------------------------\n"
	"pdf_save object me\n"
	"pdf_restore object me\n"
	"pdf_translate object me, float, float\n"
	"pdf_scale object me, float, float\n"
	"pdf_rotate object me, float\n"
	"pdf_skew object me, float, float\n"
	"pdf_concat object me, float, float, float, float, float, float\n"
	"pdf_setdash object me, float, float\n"
	"pdf_setpolydash object me, float, integer\n"
	"pdf_setflat object me, float\n"
	"pdf_setlinejoin object me, integer\n"
	"pdf_setlinecap object me, integer\n"
	"pdf_setmiterlimit object me, float\n"
	"pdf_setlinewidth object me, float\n"
	"pdf_moveto object me, float, float\n"
	"pdf_lineto object me, float, float\n"
	"pdf_curveto object me, float, float, float, float, float, float\n"
	"pdf_circle object me, float, float, float\n"
	"pdf_arc object me, float, float, float, float, float\n"
	"pdf_arcn object me, float, float, float, float, float\n"
	"pdf_rect object me, float, float, float, float\n"
	"pdf_closepath object me\n"
	"pdf_stroke object me\n"
	"pdf_closepath_stroke object me\n"
	"pdf_fill object me\n"
	"pdf_fill_stroke object me\n"
	"pdf_closepath_fill_stroke object me\n"
	"pdf_clip object me\n"
	"pdf_endpath object me\n"
};


/*****************************************************************************
 *  CREATE AND DESTROY METHODS
 *  --------------------------
 *  Every interface and class has an associated 'Create' and 'Destroy' pair.
 *  'Create' methods are typically used to acquire interface(s), allocate 
 *  memory, and intialize variables. 'Destroy' methods are typically used to 
 *  release interfaces and free memory.
 *
 * NOTE:  In C++, the local variable 'This' is provided implicitly within 
 * a method implementation.  Thus, there is no need explicitly declare 'This' 
 * as a function parameter. However, this implementation detail doesnt apply 
 * to the MOA class creator and destructor functions, which are standard C 
 * functions, coded exactly as in like they are in C examples.  
 *
 * Class Syntax:
 * STDMETHODIMP MoaCreate_<class-name>(<class-name> FAR * This)
 * STDMETHODIMP MoaDestroy_<class-name>(<class-name> FAR * This)
 *
 * Interface Syntax:
 * <class_name>_<if_name>::<class_name>_<if_name>(MoaError FAR * pErr)
 * <class_name>_<if_name>::~<class_name>_<if_name>()
 ****************************************************************************/ 

/* ---------------------------------------------------- MoaCreate_CHxRegister */
STDMETHODIMP MoaCreate_CHxRegister(CHxRegister FAR * This)
{
	UNUSED(This);
	

	MoaError		err = kMoaErr_NoErr;

	return(err);

}

/* --------------------------------------------------- MoaDestroy_CHxRegister */
STDMETHODIMP_(void) MoaDestroy_CHxRegister(CHxRegister FAR * This)
{
	UNUSED(This);

	return;
}

/* ---------------------------------- CHxRegister_IMoaRegister Create/Destroy */
CHxRegister_IMoaRegister::CHxRegister_IMoaRegister(MoaError FAR * pErr)
	{ *pErr = (kMoaErr_NoErr); }
CHxRegister_IMoaRegister::~CHxRegister_IMoaRegister() {}


/*****************************************************************************
 *  METHOD IMPLEMENTATION(S)
 *  ------------------------
 *  This is where the methods to be defined by your MOA class are implemented.
 *  The bulk of the work in implementing Xtras is done here.  NOTE: 'This' is 
 *  implemented implicitly in C++, therefore it isn't used in the argument-
 *	list.
 *
 *  Syntax:
 *  STDMETHODIMP <class-name>_<interface-name>::<method-name>(<argument-list>)
 *****************************************************************************/ 

/* ------------------------------------------- CHxRegister_IMoaRegister::Register */
STDMETHODIMP_(MoaError) CHxRegister_IMoaRegister::Register(PIMoaCache pCache, 
	PIMoaDict pXtraDict)
{

	/* variable declaration */
	MoaError 		err = kMoaErr_NoErr;
	PMoaVoid		pMemStr = NULL;
	PIMoaDict 		pRegDict = NULL;
#ifdef USING_LOAD_CONTROL
	MoaLong			loadOptions = (kMoaXlc_PreloadAtLaunch | kMoaXlc_NeverUnload);
#endif

	// This Xtra will only register in Director 6 or later
	PIMoaAppInfo	pAppInfo;
	//MoaChar			buf[64];

	if (err = pObj->pCallback->QueryInterface(&IID_IMoaAppInfo, (PPMoaVoid)&pAppInfo))
		goto exit_gracefully;
/*
	if (err = pAppInfo->GetInfo(kMoaAppInfo_ProductName, buf, 64))
		goto exit_gracefully;

	if (strcmp(buf, "Director")) 
	{
		err = kMoaErr_BadInterface;
		goto exit_gracefully;
	}
	if (err = pAppInfo->GetInfo(kMoaAppInfo_ProductVersion, buf, 64))
		goto exit_gracefully;

	if (*buf>1 && *buf < '6') 
	{
		err = kMoaErr_InternalError;
		goto exit_gracefully;
	}

	// End check for Director 6
	*/

#ifdef USING_LOAD_CONTROL
	err = pCache->AddRegistryEntry(pXtraDict, &CLSID_CPDF, &IID_IMoaXtraLoadControl, &pRegDict);
	err = pRegDict->Put(kMoaRegType_XtraLoadControlOptions, &loadOptions, 0, kMoaRegKey_XtraLoadControlOptions);
#endif

	// Register the scripting xtra
	if (err = pCache->AddRegistryEntry(pXtraDict, &CLSID_CPDF, &IID_IMoaMmXScript, &pRegDict))
		goto exit_gracefully;
	
	/* Register the method table */
	/* msgTable is too long, have to concatenate it */
	if (!(pMemStr = pObj->pCalloc->NRAlloc(sizeof(msgTable1)))) 
	{
		MOA_DEBUGSTR("calloc failed!\n");
		err = kMoaErr_OutOfMem;
		goto exit_gracefully;
	}

	strcpy((char *)pMemStr, msgTable1);

	err = pRegDict->Put(kMoaDrDictType_MessageTable, pMemStr, 0, kMoaDrDictKey_MessageTable);

exit_gracefully:

	if(pMemStr)
	{
		pObj->pCalloc->NRFree(pMemStr);
		pMemStr = NULL;
	}

	if (pAppInfo) 
	{
		pAppInfo->Release();
		pAppInfo = NULL;
	}

	return(err);
}

